/* colors.c
    Copyright (c) 1993 by Thomas E. Janzen
    All Rights Reserved

    THIS SOFTWARE IS FURNISHED FREE OF CHARGE FOR STUDY AND USE AND MAY
    BE COPIED ONLY FOR PERSONAL USE OR COMPLETELY AS OFFERED WITH NO
    CHANGES FOR FREE DISTRIBUTION.  NO TITLE TO AND OWNERSHIP OF THE
    SOFTWARE IS HEREBY TRANSFERRED.  THOMAS E. JANZEN ASSUMES NO 
    RESPONSIBILITY FOR THE USE OR RELIABILITY OF THIS SOFTWARE.
    
    Thomas E. Janzen
    208A Olde Derby Road
    Norwood, MA  02062-1761
    (617)769-7733

**  FACILITY:
**
**    AlgoRhythms music improviser on Commodore (TM) Amiga (TM)
**    compiled with SAS/C Amiga Compiler 6.50 
**
**  ABSTRACT:
**
**    colors.c supports gadgets (using GadTools) for controlling one
**    screen colors.
**
**  AUTHORS: Thomas E. Janzen
**
**  CREATION DATE:    28-DEC-1993
**
**  MODIFICATION HISTORY:
**    DATE    NAME    DESCRIPTION
**    1 JAN 94 T. Janzen New for V3.0
**-- 
*/

#define INTUI_V36_NAMES_ONLY

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <exec/types.h>
#include <exec/nodes.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>

#include <clib/exec_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/gadtools_protos.h>
#include <clib/utility_protos.h>
#include <clib/dos_protos.h>
#include <proto/graphics.h> 
#include <proto/intuition.h>
#include <proto/gadtools.h>
#include <graphics/view.h>

#include "window.h"
#include "colors.h"

#define C_PALETTE_X (10)
#define C_PALETTE_Y (16)
#define C_RED_X     (60)
#define C_RED_Y     (44)
#define C_GREEN_X   (60)
#define C_GREEN_Y   (54)
#define C_BLUE_X    (60)
#define C_BLUE_Y    (64)
#define C_SAVE_X    (75)
#define C_SAVE_Y    (80)

#define GAD_PALETTE (0)
#define GAD_RED     (1)
#define GAD_GREEN   (2)
#define GAD_BLUE    (3)
#define GAD_SAVE    (4)
#define GAD_LAST    (5)

#define SLIDER_MIN  (0)
#define SLIDER_MAX  (15)
#define C_SLIDER_HEIGHT (12)
#define C_SLIDER_WIDTH (100)
#define C_RED       (0)
#define C_GREEN     (1)
#define C_BLUE      (2)

#define C_COLOR_QTY (4)
#define C_GUN_QTY   (3)

static SHORT current_color;

static UBYTE colors[C_COLOR_QTY][C_COLOR_QTY];

static struct Window *color_wind = NULL;
static struct Gadget    *glist = NULL, 
                        *color_gads[GAD_LAST];

ULONG colors_mask = 0;

static struct Gadget *create_color_gadgets(UWORD topborder);
static void color_event(struct Gadget *gad, UWORD code);
static void set_color_gadgets(void);

static void color_event(struct Gadget *gad, UWORD code)
{
    auto int    gun, 
                entry;
    auto FILE   *color_file;

    switch (gad->GadgetID)
    {
        case GAD_PALETTE:
            current_color = code;
            set_color_gadgets();
            break;
        case GAD_RED:
            colors[current_color][C_RED]   = code;
            SetRGB4(view_port, current_color, 
              colors[current_color][C_RED], 
              colors[current_color][C_GREEN], 
              colors[current_color][C_BLUE]);
            break;
        case GAD_GREEN:
            colors[current_color][C_GREEN] = code;
            SetRGB4(view_port, current_color /* SHORT */, 
              colors[current_color][C_RED], 
              colors[current_color][C_GREEN], 
              colors[current_color][C_BLUE]);
            break;
        case GAD_BLUE:
            colors[current_color][C_BLUE]  = code;
            SetRGB4(view_port, current_color /* SHORT */, 
              colors[current_color][C_RED], 
              colors[current_color][C_GREEN], 
              colors[current_color][C_BLUE]);
            break;
        case GAD_SAVE:
            color_file = fopen(T_COLOR_FILE, "w");
            if (color_file != NULL)
            {
                for (gun = 0; gun < C_GUN_QTY; gun++)
                {
                    for (entry = 0; entry < (1 << C_WINDOW_DEPTH); entry++)
                    {
                        fprintf(color_file, "%2hd  ", colors[entry][gun]);
                    }
                    fprintf(color_file, "\n");
                }
            }
            fclose(color_file);
            break;
        default:
            break;
    }
    return;
}

static struct Gadget *create_color_gadgets(UWORD topborder)
{
    auto struct TagItem color_tag_items[] 
        = {{GTSL_Level,         0L},
           {GTSL_Min,           SLIDER_MIN}, 
           {GTSL_Max,           SLIDER_MAX},
           {GTSL_LevelFormat,   (ULONG)"%2lX"}, 
           {GTSL_MaxLevelLen,   2}, 
           {GTSL_LevelPlace,    PLACETEXT_RIGHT}, 
           {TAG_END,            0}};
    auto struct NewGadget ng;
    auto struct Gadget *gad;

    gad = CreateContext(&glist);

    ng.ng_Height = 12;
    ng.ng_TextAttr = &font_choice;
    ng.ng_VisualInfo = vi;

    /*
    ** Set up color selection
    */
    ng.ng_LeftEdge = C_PALETTE_X;
    ng.ng_TopEdge = C_PALETTE_Y;
    ng.ng_Width = 150;
    ng.ng_Height = 24;
    ng.ng_GadgetText = "Palette";
    ng.ng_GadgetID = GAD_PALETTE;
    ng.ng_Flags = NG_HIGHLABEL | PLACETEXT_LEFT;
    color_gads[GAD_PALETTE]
      = gad = CreateGadget(PALETTE_KIND, gad, &ng, 
        GTPA_Depth, C_WINDOW_DEPTH, 
        GTPA_IndicatorWidth, 20, /* GTPA_IndicatorHeight, 20,*/
        TAG_END); 
          
    ng.ng_LeftEdge  = C_RED_X;
    ng.ng_TopEdge   = C_RED_Y;
    ng.ng_Width     = C_SLIDER_WIDTH;
    ng.ng_Height    = C_SLIDER_HEIGHT;
    ng.ng_GadgetText = "RED";
    ng.ng_GadgetID  = GAD_RED;
    color_gads[GAD_RED] = gad = 
        CreateGadgetA(SLIDER_KIND, gad, &ng, color_tag_items);

    ng.ng_LeftEdge  = C_GREEN_X;
    ng.ng_TopEdge   = C_GREEN_Y;
    ng.ng_Width     = C_SLIDER_WIDTH;
    ng.ng_Height    = C_SLIDER_HEIGHT;
    ng.ng_GadgetText = "GREEN";
    ng.ng_GadgetID  = GAD_GREEN;
    color_gads[GAD_GREEN] = gad = 
        CreateGadgetA(SLIDER_KIND, gad, &ng, color_tag_items);

    ng.ng_LeftEdge  = C_BLUE_X;
    ng.ng_TopEdge   = C_BLUE_Y;
    ng.ng_Width     = C_SLIDER_WIDTH;
    ng.ng_Height    = C_SLIDER_HEIGHT;
    ng.ng_GadgetText = "BLUE";
    ng.ng_GadgetID  = GAD_BLUE;
    color_gads[GAD_BLUE] = gad = 
        CreateGadgetA(SLIDER_KIND, gad, &ng, color_tag_items);

    ng.ng_LeftEdge  = C_SAVE_X;
    ng.ng_TopEdge   = C_SAVE_Y;
    ng.ng_Width     = 48;
    ng.ng_Height    = 12;
    ng.ng_GadgetText = "SAVE";
    ng.ng_Flags     = NG_HIGHLABEL | PLACETEXT_IN;
    ng.ng_GadgetID  = GAD_SAVE;
    color_gads[GAD_SAVE] = gad 
        = CreateGadgetA(BUTTON_KIND, gad, &ng, TAG_END);

    return gad;
}

int process_color_events(void)
{
    auto int result = 0,
             Closing = FALSE;
    auto ULONG imsgClass;
    auto UWORD imsgCode;
    auto struct Gadget *gad;
    auto struct IntuiMessage *imsg;
    
    while (imsg = GT_GetIMsg(color_wind->UserPort))
    {
        gad = (struct Gadget *)imsg->IAddress;
        imsgClass = imsg->Class;
        imsgCode = imsg->Code;
        GT_ReplyIMsg(imsg);
        switch(imsgClass)
        {
            case IDCMP_CLOSEWINDOW:
                Closing = TRUE;
                break;
            case IDCMP_GADGETDOWN:
            case IDCMP_MOUSEMOVE:
            case IDCMP_GADGETUP:
                color_event(gad, imsgCode);
                result = 1;
                break;
            case IDCMP_REFRESHWINDOW:
                GT_BeginRefresh(color_wind);
                GT_EndRefresh(color_wind, TRUE);
                break;
        }
    }
    if (Closing)
    {
        close_colors();
    }
    return result;
}

void open_colors_window(void)
{
    auto UWORD   topborder;
    static struct TagItem color_tags[] = {
      {WA_CustomScreen, NULL}, 
      {WA_Title,      (ULONG)"COLORS"},
      {WA_Left,        200L},
      {WA_AutoAdjust,  TRUE},
      {WA_Width,       190}, 
      {WA_MinWidth,    290},
      {WA_InnerHeight, 86}, 
      {WA_MinHeight,   98},
      {WA_DragBar,     TRUE}, 
      {WA_DepthGadget, TRUE},
      {WA_Activate,    TRUE}, 
#if 0
      {WA_CloseGadget, TRUE},
#endif
      {WA_SimpleRefresh, TRUE},
      {WA_IDCMP, IDCMP_CLOSEWINDOW | IDCMP_REFRESHWINDOW 
            | SLIDERIDCMP | PALETTEIDCMP},
      {TAG_END, NULL}};    
        
    color_tags[0].ti_Data = (ULONG)main_screen;
    if (color_wind != NULL) 
    {
        return;
    }
    topborder   = main_screen->WBorTop
                + (main_screen->Font->ta_YSize + 1);
    create_color_gadgets(topborder);
    
    if (NULL == (color_wind = OpenWindowTagList(NULL, color_tags)))
    {
         ;
    }
    else
    {
        AddGList(color_wind, glist, -1, -1, NULL);
        RefreshGList(glist, color_wind, NULL, -1);
        GT_RefreshWindow(color_wind, NULL);
        set_color_gadgets();
        RefreshGList(glist, color_wind, NULL, -1);
        colors_mask = 1 << color_wind->UserPort->mp_SigBit;
    }
    return;
}

void close_colors(void)
{
    if (color_wind != NULL) {
        RemoveGList(color_wind, glist, -1);
        CloseWindowSafely(color_wind);
        color_wind = NULL;
        FreeGadgets(glist);
        glist = NULL;
    }
    return;
}

static void set_color_gadgets(void)
{
    auto int entry;
    auto UWORD rgb;

    for (entry = 0; entry < (1 << C_WINDOW_DEPTH); entry++)
    {
        rgb = GetRGB4(view_port->ColorMap, entry);
        colors[entry][C_RED]   = (SHORT)((rgb >> 8) & 0xF);
        colors[entry][C_GREEN] = (SHORT)((rgb >> 4) & 0xF);
        colors[entry][C_BLUE]  = (SHORT)(rgb & 0xF);
        if (entry == current_color)
        {
            RemoveGadget(color_wind, color_gads[GAD_RED]);
            RemoveGadget(color_wind, color_gads[GAD_BLUE]);
            RemoveGadget(color_wind, color_gads[GAD_GREEN]);
            GT_SetGadgetAttrs(color_gads[GAD_RED], color_wind, NULL,
                GTSL_Level, (SHORT)((rgb >> 8) & 0xF), TAG_END, 0L);
            GT_SetGadgetAttrs(color_gads[GAD_GREEN], color_wind, NULL,
                GTSL_Level, (SHORT)((rgb >> 4) & 0xF), TAG_END, 0L);
            GT_SetGadgetAttrs(color_gads[GAD_BLUE], color_wind, NULL,
                GTSL_Level, (SHORT)(rgb & 0xF), TAG_END, 0L);
            AddGadget(color_wind, color_gads[GAD_RED], -1);
            AddGadget(color_wind, color_gads[GAD_GREEN], -1);
            AddGadget(color_wind, color_gads[GAD_BLUE], -1);
        }
    }
    RefreshGadgets(glist, color_wind, NULL);
    return;
}

